#!/usr/bin/perl -w
##
## (c)  Copyright  IBM Corp.  2004  All Rights Reserved
##
## US Government Users Restricted Rights - Use, duplication or
## disclosure restricted by GSA ADP Schedule Contract with IBM Corp
##
## Module History:
##  -00 10/29/2004 John Thornton   D61C initial release
##  -01 03/31/2005 John Thornton   D61S link processing
##
##

#
# Includes
#
use Getopt::Long;

##############################################################################
#####                          GLOBAL VARIABLES                          #####
##############################################################################

# Trace Constants
$TRC_T = "XMCFSPST:";
$TRC_D = "XMCFSPSD:";
$TRC_F = "XMCFSPSF:";

# Error Constants
$ERROR_NO_ERROR               =  0;
$ERROR_NOT_ROOT               =  1;
$ERROR_CANT_EXECUTE_SWAP      =  2;
$ERROR_UTILS_NOT_FOUND        =  3;
$ERROR_PERMS_FILE_ACCESS      =  4;
$ERROR_PERMS_CONTENT_ERROR    =  5;
$ERROR_PERMS_NO_ACTION        =  6;
$ERROR_PERMS_OWNER_FAILURE    =  7;
$ERROR_PERMS_GROUP_FAILURE    =  8;
$ERROR_PERMS_FLAGS_FAILURE    =  9;
# start -01
$ERROR_LINKS_FILE_ACCESS      = 10;
$ERROR_LINKS_CONTENT_ERROR    = 11;
$ERROR_LINKS_BAD_COMMAND      = 12;
$ERROR_LINKS_NO_ALIAS         = 13;
$ERROR_LINKS_REMOVE_FAILURE   = 14;
$ERROR_LINKS_BAD_TYPE         = 15;
$ERROR_LINKS_NO_TARGET        = 16;
$ERROR_LINKS_ALIAS_EXISTS     = 17;
$ERROR_LINKS_CREATE_FAILURE   = 18;
# end -01
$ERROR_HELP                   = 99;

# Boolean Constants
$TRUE  = 1;
$FALSE = 0;

# Program Data
$me = $0;
$me =~ s/^(.*)\///;
$me =~ s/.pl$//;
$traceCmd = "actzTrace";

##############################################################################
#####                          SHARED METHODS                            #####
##############################################################################

#
#
#
sub queryFileLocation
{
   my $this = $me . ".queryFileLocation()";
   my $file = shift;
   trace("$TRC_T -> $this - $file");
   my $consoleDir;
   # Many possibilities in the non-production world
   if ($ENV{'CONSOLE_PATH'})
   {
      $consoleDir = $ENV{'CONSOLE_PATH'}.'/';
      trace("$TRC_F    $this - used CONSOLE_PATH");
   }
   elsif ($ENV{'HWMCAHOME'})
   {
      $consoleDir = $ENV{'HWMCAHOME'}.'/';
      trace("$TRC_F    $this - used HWMCAHOME");
   }
   else
   {
      $consoleDir = "/console/";
      trace("$TRC_F    $this - used default");
   }
   # Change any "//" to "/"
   $consoleDir =~ s/\/+/\//g;
   trace("$TRC_F    $this - console root is $consoleDir");
   if ( -x "$consoleDir/hmcfunctions" )
   {
     $path = `. hmcfunctions; queryFileLocation $file`;
     # Remove eol
     chomp($path);
   }
   else
   {
     # Make our best guess
     $path = $consoleDir . "data/";
     trace("$TRC_F    $this - hmcfunctions not found - only guessing here");
     my $hex_rc = sprintf("0x%08X", $ERROR_UTILS_NOT_FOUND);
     `logerror 0xAA 0x02 0x0000 0xE562 $hex_rc`;
   }
   trace("$TRC_T <- $this - $path");
   return($path);
}

#
#
#
sub setHozd
{
   my $this = $me . ".setHosed()";
   my $msg = shift;
   trace("$TRC_T -> $this - message is $msg");

   my $rc = $ERROR_NO_ERROR;
   my $filename = "iqzmhozd.dat";

   my $path = queryFileLocation($filename);
   my $hozdfile = $path . $filename;
   # Change any "//" to "/"
   $hozdfile =~ s/\/+/\//g;
   trace("$TRC_F    $this - hosed file is $hozdfile");

   open(HOZDFILE, ">> " . $hozdfile);
   print HOZDFILE ($msg . "\n");
   close(HOZDFILE);

   trace("$TRC_F    $this - issue the HOZD refcode");
   `logerror 0xAA 0x02 0x0000 0xE563 0x0000DEAD`;

   trace("$TRC_T <- $this");
   return;
}

#
#
#
sub trace
{
    my $text = shift;
    chomp($text);

    if ("actzTrace" eq "$traceCmd")
    {
       system(("$traceCmd", "$text"));
    }
    else
    {
       print(STDOUT "$text\n");
    }
}

#
# usage
#
sub usage
{
    print(<<EOM);
Usage: iqzmShutdownProcessing [-help] [-tracecommand=cmd]
       this command is intended to be run from the manager script as the
       manager is shutting down
EOM
}

##############################################################################
#####                           FILE SWAPPING                            #####
##############################################################################

#
#
#
sub doFileSwapping
{
   my $this = $me . ".doFileSwapping()";
   trace("$TRC_T -> $this");

   my $rc = $ERROR_NO_ERROR;
   my $filename = "iqzmlock.sh";

   my $path = queryFileLocation($filename);
   my $swapfile = $path . $filename;
   # Change any "//" to "/"
   $swapfile =~ s/\/+/\//g;
   trace("$TRC_F    $this - swap file script is $swapfile");

   # Does the file exist?
   if ( -f "$swapfile" )
   {
      # Ensure the file is executable
      if ( !( -x "$swapfile" ))
      {
         # Not yet - set the permission bits
         my $count = chmod(0755, ($swapfile));
         if ($count < 1)
         {
            # chmod() failed, so we are toast for running the file.
            # This is fatal.
            trace("$TRC_F    $this - Unable to execute $swapfile");
            $rc = $ERROR_CANT_EXECUTE_SWAP;
            my $hex_rc = sprintf("0x%08X", $rc);
            `logerror 0xAA 0x02 0x0000 0xE561 $hex_rc`;
            setHozd("$this - Unable to execute $swapfile");
         }
      }

      if ($rc == $ERROR_NO_ERROR)
      {
         # Invoke the swap script
         trace("$TRC_F    $this - Running $swapfile");
         print(`$swapfile`);
      }
   }
   else
   {
      # There is nothing to do
      trace("$TRC_F    $this - No file swapping data was found");
   }

   trace("$TRC_T <- $this - rc = $rc");
   return($rc);
}

##############################################################################
#####                          LINK PROCESSING                           #####
##############################################################################

#
#
# start -01
#
sub doLinkProcessing
{
   my $this = $me . ".doLinkProcessing()";
   trace("$TRC_T -> $this");

   my $rc = $ERROR_NO_ERROR;
   my $filename = "iqzmLinks.dat";

   my $path = queryFileLocation($filename);
   my $datafile = $path . $filename;
   # Change any "//" to "/"
   $datafile =~ s/\/+/\//g;
   trace("$TRC_F    $this - links data file is $datafile");

   # Does the file exist?
   if ( -f "$datafile" )
   {
      unless (open(DATAFILE, $datafile))
      {
         trace("$TRC_F    $this - error opening links data file");
         $rc = $ERROR_LINKS_FILE_ACCESS;
      }

      if ($rc == $ERROR_NO_ERROR)
      {
         trace("$TRC_D    $this - reading links data file");
         my @mydata = <DATAFILE>;

         trace("$TRC_D    $this - closing links data file");
         close (DATAFILE);

         $rc = processLinks(@mydata);
      }

      # Copy file to .old version
      my $target = $datafile . ".old";
      `cp -f $datafile $target`;

      # Copy file to .err version if error
      if ($rc != $ERROR_NO_ERROR)
      {
         $target = $datafile . ".err";
         `cp -f $datafile $target`;
      }

      # Erase the data file
      `rm $datafile`;

      if ($rc != $ERROR_NO_ERROR)
      {
         # Our processing failed, so we may not be able to access the files
         # where links were to be changed.
         # This is fatal.
         trace("$TRC_F    $this - Failure updating file links");
         my $hex_rc = sprintf("0x%08X", $rc);
         `logerror 0xAA 0x02 0x0000 0xE566 $hex_rc`;
         setHozd("$this - Unable to update file links");
      }
   }
   else
   {
      # There is nothing to do
      trace("$TRC_F    $this - No links data file was found");
   }

   trace("$TRC_T <- $this - rc = $rc");
   return($rc);
}

#
#
#
sub processLinks
{
   my $this = $me . ".processLinks()";
   trace("$TRC_T -> $this");

   my $rc = $ERROR_NO_ERROR;

   my @lines = @_;

   my $command = "";
   my $alias   = "";
   my $target  = "";
   my $type    = "";

   my $key     = "";
   my $value   = "";
   my $line    = "";

   my $contentError = $FALSE;

   foreach $line (@lines)
   {
      # Remove the eol
      chomp($line);
      trace("$TRC_D    $this - line is [$line]");

      # Break the line into a key and its value
      ($key, $value) = $line =~ /^<(.*?)>(.*)/;
      # These elements can end up undefined, so handle that to
      # avoid runtime error messages
      unless (defined($key))   { $key = "";   }
      unless (defined($value)) { $value = ""; }

      trace("$TRC_D    $this - key is [$key], value is [$value]");
      if ($key eq "cmd")
      {
         trace("$TRC_D    $this - command is [$value]");
         my $newcommand = $value;

         # process the data for the previous file
         if ($command ne "")
         {
            my $trc = processOneLink($command, $alias, $target, $type);
            # Save the rc if it represents an error
            if ($trc != $ERROR_NO_ERROR)
            {
               $rc = $trc
            }
         }
         $command = $newcommand;
         $alias   = "";
         $target  = "";
         $type    = "";
      }
      elsif ($key eq "alias")
      {
         trace("$TRC_D    $this - alias is [$value]");
         my $newalias = $value;
         if ($command eq "")
         {
            trace("$TRC_F    $this - no command defined - alias value ignored [$newalias]");
            $contentError = $TRUE;
         }
         else
         {
            if ($alias ne "")
            {
               trace("$TRC_F    $this - multiple alias records for $command - using last value [$newalias]");
               $contentError = $TRUE;
            }
            $alias = $newalias;
         }
      }
      elsif ($key eq "target")
      {
         trace("$TRC_D    $this - target is [$value]");
         my $newtarget = $value;
         if ($command eq "")
         {
            trace("$TRC_F    $this - no command defined - target value ignored [$newtarget]");
            $contentError = $TRUE;
         }
         else
         {
            if ($target ne "")
            {
               trace("$TRC_F    $this - multiple target records for $command - using last value [$newtarget]");
               $contentError = $TRUE;
            }
            $target = $newtarget;
         }
      }
      elsif ($key eq "type")
      {
         trace("$TRC_D    $this - type are [$value]");
         my $newtype = $value;
         if ($command eq "")
         {
            trace("$TRC_F    $this - no command defined - type value ignored [$newtype]");
            $contentError = $TRUE;
         }
         else
         {
            if ($type ne "")
            {
               trace("$TRC_F    $this - multiple type records for $command - using last value [$newtype]");
               $contentError = $TRUE;
            }
            $type = $newtype;
         }
      }
      else
      {
         trace("$TRC_F    $this - unrecognized line [$line]");
         $contentError = $TRUE;
      }
   }

   # process the data for the previous (last) command
   if ($command ne "")
   {
      my $trc = processOneLink($command, $alias, $target, $type);
      # Save the rc if it represents an error
      if ($trc != $ERROR_NO_ERROR)
      {
         $rc = $trc
      }
   }

   # If there has been no other error so far, make a note of any content error
   if ($rc == $ERROR_NO_ERROR)
   {
      if ($contentError == $TRUE)
      {
         $rc = $ERROR_LINKS_CONTENT_ERROR;
      }
   }

   # Any logging is done at the top level of link processing

   trace("$TRC_T <- $this - rc = $rc");
   return($rc);
}

#
#
#
sub processOneLink
{
   my $this = $me . ".processOneLink()";
   trace("$TRC_T -> $this");

   my $rc = $ERROR_NO_ERROR;

   my ($command, $alias, $target, $type) = @_;

   trace("$TRC_F    $this - parms are [$command], [$alias], [$target], [$type]");

   # Check the command

   if ($command eq "create")
   {
      # Call the subroutine to do create processing
      my $rc = createOneLink($alias, $target, $type);
   }
   elsif ($command eq "remove")
   {
      # Call the subroutine to do remove processing
      my $rc = removeOneLink($alias);
   }
   else
   {
      trace("$TRC_F    $this - command ($command) is not recognized");
      $rc = $ERROR_LINKS_BAD_COMMAND;
   }

   # Any logging is done at the top level of link processing

   trace("$TRC_T <- $this - rc = $rc");
   return($rc);
}

#
#
#
sub removeOneLink
{
   my $this = $me . ".removeOneLink()";
   trace("$TRC_T -> $this");

   my $rc = $ERROR_NO_ERROR;

   my ($alias) = @_;

   trace("$TRC_F    $this - parms are [$alias]");

   # Ensure the target exists
   if ( -f "$alias" )
   {
      my $error = `rm -f $alias 2>&1`;
      unless (defined($error)) { $error = ""; }
      chop $error;
      $rc = $? >> 8;
      if ($rc)
      {
         trace("$TRC_F    $this - rm error deleting link - \"$error\"");
         $rc = $ERROR_LINKS_REMOVE_FAILURE;
      }
   }
   else
   {
      # There is nothing to do
      trace("$TRC_F    $this - No alias was found to remove");
      $rc = $ERROR_LINKS_NO_ALIAS;
   }

   # Any logging is done at the top level of link processing

   trace("$TRC_T <- $this - rc = $rc");
   return($rc);
}

#
#
#
sub createOneLink
{
   my $this = $me . ".createOneLink()";
   trace("$TRC_T -> $this");

   my $rc = $ERROR_NO_ERROR;

   my ($alias, $target, $type) = @_;
   my $opts = "";

   trace("$TRC_F    $this - parms are [$alias], [$target], [$type]");

   # Check for symbolic or hard link
   if ( $type eq "911" )
   {
      $opts = "-s";
   }
   elsif ( $type eq "912" )
   {
      $opts = "";
   }
   else
   {
      trace("$TRC_F    $this - unrecognized link type value: $type");
      $rc = $ERROR_LINKS_BAD_TYPE;
   }

   # Ensure the target exists
   if ( -f "$target" )
   {
      if ( -e "$alias" )
      {
         # There is already some file present where the alias is
         # supposed to go
         trace("$TRC_F    $this - File already exists where alias should go");
         $rc = $ERROR_LINKS_ALIAS_EXISTS;
      }
      else
      {
         my $error = `ln $opts $target $alias 2>&1`;
         unless (defined($error)) { $error = ""; }
         chop $error;
         $rc = $? >> 8;
         if ($rc)
         {
            trace("$TRC_F    $this - ln error creating link - \"$error\"");
            $rc = $ERROR_LINKS_CREATE_FAILURE;
         }
      }
   }
   else
   {
      # The target doesn't exist
      trace("$TRC_F    $this - The link target does not exist");
      $rc = $ERROR_LINKS_NO_TARGET;
   }

   # Any logging is done at the top level of link processing

   trace("$TRC_T <- $this - rc = $rc");
   return($rc);
}
# end -01

##############################################################################
#####                        PERMISSION CHANGING                         #####
##############################################################################

#
#
#
sub doPermissionChanging
{
   my $this = $me . ".doPermissionChanging()";
   trace("$TRC_T -> $this");

   my $rc = $ERROR_NO_ERROR;
   my $filename = "iqzmPermissions.dat";

   my $path = queryFileLocation($filename);
   my $datafile = $path . $filename;                                            #-01
   # Change any "#" to "/"
   $datafile =~ s/\/+/\//g;                                                     #-01
   trace("$TRC_F    $this - permissions data file is $datafile");               #-01

   # Does the file exist?
   if ( -f "$datafile" )                                                        #-01
   {
      unless (open(DATAFILE, $datafile))                                        #-01
      {
         trace("$TRC_F    $this - error opening permissions data file");
         $rc = $ERROR_PERMS_FILE_ACCESS;
      }

      if ($rc == $ERROR_NO_ERROR)
      {
         trace("$TRC_D    $this - reading permissions data file");
         my @mydata = <DATAFILE>;                                               #-01

         trace("$TRC_D    $this - closing permissions data file");
         close (DATAFILE);                                                      #-01

         $rc = processPermissions(@mydata);                                     #-01
      }

      # Copy file to .old version
      my $target = $datafile . ".old";                                          #-01
      `cp -f $datafile $target`;                                                #-01

      # Copy file to .err version if error
      if ($rc != $ERROR_NO_ERROR)
      {
         $target = $datafile . ".err";                                          #-01
         `cp -f $datafile $target`;                                             #-01
      }

      # Erase the data file
      `rm $datafile`;                                                           #-01

      if ($rc != $ERROR_NO_ERROR)
      {
         # Our processing failed, so we may not be able to access the files
         # where permissions were to be changed.
         # This is fatal.
         trace("$TRC_F    $this - Failure updating file permissions");
         my $hex_rc = sprintf("0x%08X", $rc);
         `logerror 0xAA 0x02 0x0000 0xE564 $hex_rc`;
         setHozd("$this - Unable to update file permissions");
      }
   }
   else
   {
      # There is nothing to do
      trace("$TRC_F    $this - No permissions data file was found");
   }

   trace("$TRC_T <- $this - rc = $rc");
   return($rc);
}

#
#
#
sub processPermissions
{
   my $this = $me . ".processPermissions()";
   trace("$TRC_T -> $this");

   my $rc = $ERROR_NO_ERROR;

   my @lines = @_;

   my $file    = "";
   my $owner   = "";
   my $group   = "";
   my $flags   = "";
   my $key     = "";
   my $value   = "";

   my $contentError = $FALSE;

   foreach $line (@lines)
   {
      # Remove the eol
      chomp($line);
      trace("$TRC_D    $this - line is [$line]");

      # Break the line into a key and its value
      ($key, $value) = $line =~ /^<(.*?)>(.*)/;
      # These elements can end up undefined, so handle that to
      # avoid runtime error messages
      unless (defined($key))   { $key = "";   }
      unless (defined($value)) { $value = ""; }

      trace("$TRC_D    $this - key is [$key], value is [$value]");
      if ($key eq "file")
      {
         trace("$TRC_D    $this - file is [$value]");
         my $newfile = $value;

         # process the data for the previous file
         if ($file ne "")
         {
            my $trc = processPermissionsForOneFile($file, $owner, $group, $flags);
            # Save the rc if it represents an error
            if ($trc != $ERROR_NO_ERROR)
            {
               $rc = $trc
            }
         }
         $owner   = "";
         $group   = "";
         $flags   = "";
         $file = $newfile;

      }
      elsif ($key eq "owner")
      {
         trace("$TRC_D    $this - owner is [$value]");
         my $newowner = $value;
         if ($file eq "")
         {
            trace("$TRC_F    $this - no file defined - owner value ignored [$newowner]");
            $contentError = $TRUE;
         }
         else
         {
            if ($owner ne "")
            {
               trace("$TRC_F    $this - multiple owner records for $file - using last value [$newowner]");
               $contentError = $TRUE;
            }
            $owner = $newowner;
         }
      }
      elsif ($key eq "group")
      {
         trace("$TRC_D    $this - group is [$value]");
         my $newgroup = $value;
         if ($file eq "")
         {
            trace("$TRC_F    $this - no file defined - group value ignored [$newgroup]");
            $contentError = $TRUE;
         }
         else
         {
            if ($group ne "")
            {
               trace("$TRC_F    $this - multiple group records for $file - using last value [$newgroup]");
               $contentError = $TRUE;
            }
            $group = $newgroup;
         }
      }
      elsif ($key eq "flags")
      {
         trace("$TRC_D    $this - flags are [$value]");
         my $newflags = $value;
         if ($file eq "")
         {
            trace("$TRC_F    $this - no file defined - flags value ignored [$newflags]");
            $contentError = $TRUE;
         }
         else
         {
            if ($flags ne "")
            {
               trace("$TRC_F    $this - multiple flags records for $file - using last value [$newflags]");
               $contentError = $TRUE;
            }
            $flags = $newflags;
         }
      }
      else
      {
         trace("$TRC_F    $this - unrecognized line [$line]");
         $contentError = $TRUE;
      }
   }

   # process the data for the previous (last) file
   if ($file ne "")
   {
      my $trc = processPermissionsForOneFile($file, $owner, $group, $flags);
      # Save the rc if it represents an error
      if ($trc != $ERROR_NO_ERROR)
      {
         $rc = $trc
      }
   }

   # If there has been no other error so far, make a note of any content error
   if ($rc == $ERROR_NO_ERROR)
   {
      if ($contentError == $TRUE)
      {
         $rc = $ERROR_PERMS_CONTENT_ERROR;
      }
   }

   trace("$TRC_T <- $this - rc = $rc");
   return($rc);
}

#
#
#
sub processPermissionsForOneFile
{
   my $this = $me . ".processPermissionsForOneFile()";
   trace("$TRC_T -> $this");

   my $rc = $ERROR_NO_ERROR;

   my ($file, $owner, $group, $flags) = @_;

   trace("$TRC_F    $this - parms are [$file], [$owner], [$group], [$flags]");

   my $actionRequired = $FALSE;

   # Change the owner
   if ($owner ne "")
   {
      $actionRequired = $TRUE;
      my $cmd = "chown $owner $file 2>&1";
      my $result = `$cmd`;
      my $trc = $? >> 8;
      if ($trc != 0)
      {
         # Had a problem. Shouldn't happen. This is bad.
         trace("$TRC_F    $this - command is [$cmd], rc = $trc, result is [$result]");
         $rc = $ERROR_PERMS_OWNER_FAILURE;
         my $hex_rc = sprintf("0x%08X", $rc);
         `logerror 0xAA 0x02 0x0000 0xE565 $hex_rc`;
      }
      else
      {
         trace("$TRC_D    $this - command is [$cmd], result is [$result]");
      }
   }

   # Change the group
   if ($rc == $ERROR_NO_ERROR)
   {
      if ($group ne "")
      {
         $actionRequired = $TRUE;
         my $cmd = "chown :$group $file 2>&1";
         my $result = `$cmd`;
         my $trc = $? >> 8;
         if ($trc != 0)
         {
            # Had a problem. Shouldn't happen. This is bad.
            trace("$TRC_F    $this - command is [$cmd], rc = $trc, result is [$result]");
            $rc = $ERROR_PERMS_GROUP_FAILURE;
            my $hex_rc = sprintf("0x%08X", $rc);
            `logerror 0xAA 0x02 0x0000 0xE565 $hex_rc`;
         }
         else
         {
            trace("$TRC_D    $this - command is [$cmd], result is [$result]");
         }
      }
   }

   # Change the permission bits
   if ($rc == $ERROR_NO_ERROR)
   {
      if ($flags ne "")
      {
         $actionRequired = $TRUE;
         my $cmd = "chmod $flags $file 2>&1";
         my $result = `$cmd`;
         my $trc = $? >> 8;
         if ($trc != 0)
         {
            # Had a problem. Shouldn't happen. This is bad.
            trace("$TRC_F    $this - command is [$cmd], rc = $trc, result is [$result]");
            $rc = $ERROR_PERMS_FLAGS_FAILURE;
            my $hex_rc = sprintf("0x%08X", $rc);
            `logerror 0xAA 0x02 0x0000 0xE565 $hex_rc`;
         }
         else
         {
            trace("$TRC_D    $this - command is [$cmd], result is [$result]");
         }
      }
   }

   # Was there something to do?
   if ($actionRequired == $FALSE)
   {
      trace("$TRC_F    $this - no action was specified for file $file");
      my $hex_rc = sprintf("0x%08X", $ERROR_PERMS_NO_ACTION);
      `logerror 0xAA 0x02 0x0000 0xE565 $hex_rc`;
      # This is not a fatal error.
   }

   trace("$TRC_T <- $this - rc = $rc");
   return($rc);
}

##############################################################################
#####                          MAINLINE CODE                             #####
##############################################################################

# Process the arguments
GetOptions(
      'help'            =>\$help,
      'tracecommand:s'  =>\$traceCmd,
);

unless (defined($traceCmd )) { $traceCmd = ""; }                                #-01
                                                                                #-01
trace("$TRC_T -> $me(@ARGV)");

my $rc = $ERROR_NO_ERROR;

unless($> == 0)
{
   trace("$TRC_F    $me() - this must be run with root authority");
   my $hex_rc = sprintf("0x%08X", $ERROR_NOT_ROOT);
   `logerror 0xAA 0x02 0x0000 0xE560 $hex_rc`;
}

if ($help)
{
   usage();
   $rc = $ERROR_HELP;
}

# Run each of the functions, unless an error interferes.

if ($rc == $ERROR_NO_ERROR)
{
   $rc = doFileSwapping();
}

if ($rc == $ERROR_NO_ERROR)
{
   $rc = doLinkProcessing();
}

if ($rc == $ERROR_NO_ERROR)
{
   $rc = doPermissionChanging();
}

trace("$TRC_T <- $me() - rc = $rc");

exit($rc);

